package net.pingex.dcf.commands.parser;

import net.pingex.dcf.core.Configuration;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Basic command parser.
 */
public class BasicParser implements ICommandParser
{
	private Pattern commandPattern = Pattern.compile("^" + Configuration.COMMAND_PREFIX + "([:\\w]+)(?: (.*))?$");
	private Pattern argsPattern = Pattern.compile("([^\"]\\S*|\".+?\")\\s*");

	@Override
	public boolean checkSyntax(String input)
	{
		return commandPattern.matcher(input).matches();
	}

	@Override
	public String parseCommand(String input) throws ParserException
	{
		Matcher m = commandPattern.matcher(input);
		if(!m.matches()) throw new ParserException("String cannot be parsed for a command.");
		return m.group(1);
	}

	@Override
	public List<String> parseArguments(String input) throws ParserException
	{
		Matcher mc = commandPattern.matcher(input);
		if(!mc.matches()) throw new ParserException("String cannot be parsed for a command.");

		// CASE: Command has no arg
		if(mc.group(2) == null) return Collections.emptyList();

		// CASE: Command has args
		Matcher ma = argsPattern.matcher(mc.group(2));
		List<String> argsOut = new ArrayList<>();
		while(ma.find()) argsOut.add(ma.group(1).replace("\"", ""));
		return argsOut;
	}
}
