package net.pingex.dcf.modularity;

import java.net.URLClassLoader;

/**
 * A class wrapping a {@code IPlugin} class.
 */
public class PluginWrapper
{
	/**
	 * Unique ID of the plugin.
	 */
	private String id;

	/**
	 * Version of this plugin.
	 */
	private String version;

	/**
	 * Plugin short description
	 */
	private String description;

	/**
	 * Matching plugin class.
	 */
	private Class<? extends IPlugin> pluginClass;

	/**
	 * The associated class loader.
	 */
	private URLClassLoader classLoader;

	/**
	 * The plugin main instance.
	 */
	private IPlugin instance;

	/**
	 * The current state of this plugin.
	 */
	private PluginState state;

	public PluginWrapper(Class<? extends IPlugin> pluginClass, URLClassLoader classLoader) throws IllegalAccessException, InstantiationException
	{
		this.id = pluginClass.getAnnotation(Plugin.class).id();
		this.version = pluginClass.getAnnotation(Plugin.class).version();
		this.description = pluginClass.getAnnotation(Plugin.class).description();
		this.pluginClass = pluginClass;
		this.classLoader = classLoader;
		this.instance = pluginClass.newInstance();
		this.state = PluginState.UNLOADED;
	}

	// GETTERS

	public String getId()
	{
		return id;
	}

	public String getVersion()
	{
		return version;
	}

	public String getDescription()
	{
		return description;
	}

	public Class<? extends IPlugin> getPluginClass()
	{
		return pluginClass;
	}

	public URLClassLoader getClassLoader()
	{
		return classLoader;
	}

	public IPlugin getInstance()
	{
		return instance;
	}

	public PluginState getState()
	{
		return state;
	}

	// SETTERS

	public void setInstance(IPlugin instance)
	{
		this.instance = instance;
	}

	public void setState(PluginState state)
	{
		this.state = state;
	}

	@Override
	public boolean equals(Object obj)
	{
		return obj instanceof PluginWrapper && id.equals(((PluginWrapper) obj).getId());
	}
}
