package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Command;
import sx.blah.discord.handle.obj.IGuild;
import sx.blah.discord.handle.obj.IRole;
import sx.blah.discord.handle.obj.IUser;

/**
 * Provider of permissions, ie. the proxy between {@code PermissionsHandler} and a data storage.
 */
public interface ICommandPermissionsProvider
{
	/**
	 * Validate individual user permission.
	 * @param guild Guild where command is executed. `null` = globally
	 * @param user Target user
	 * @param command Command to test
	 * @return `true` if the user is able to run the command, `false` if he isn't able to, `null` if there is no answer.
	 */
	Boolean validateUser(IGuild guild, IUser user, Command command);

	/**
	 * Validate a role's ability to run a command.
	 * @param role Target role
	 * @param command Command to test
	 * @return `true` if the role is able to run the command, `false` if he isn't able to, `null` if there is no answer.
	 */
	Boolean validateGroup(IRole role, Command command);

	/**
	 * Set an user's permission to access a command.
	 * @param guild On which guild does the permission apply ? Set to `null` for global range.
	 * @param user Target user.
	 * @param command Target command.
	 * @param value `null` to remove the key, `false` to deny, `true` to allow.
	 */
	void setUserPermission(IGuild guild, IUser user, Command command, Boolean value);

	/**
	 * Set an group's permission to access a command.
	 * @param role Target role.
	 * @param command Target command.
	 * @param value `null` to remove the key, `false` to deny, `true` to allow.
	 */
	void setGroupPermissions(IRole role, Command command, Boolean value);
}
